require 'spec_helper'

describe Buyatabwrapper::API do
  subject { Buyatabwrapper::API.new }

  class Rails
    def self.root
      Pathname.new("./spec/").realpath
    end

    def self.env
      'test'
    end
  end

  describe '.send_request', :vcr do
    context 'with configs' do
      context 'for purchase_card call' do
        let(:params) {{ merchant_id: '22', amount: '10.00' }}

        it 'returns successful result' do
          expect(subject.send_request(:purchase_card, params).successful?).to be_truthy
        end

        it 'returns transaction id' do
          expect(subject.send_request(:purchase_card, params).response.id).to eq(12345)
        end

        it 'returns link to card' do
          expect(subject.send_request(:purchase_card, params).response.url).to eq('https://www.buyatab.com/gcp/ecard.aspx?Id=JjD4DcN3vDv7IINuKFrweg%3d%3d')
        end

        it 'returns card number' do
          expect(subject.send_request(:purchase_card, params).response.card_number).to eq('1234567890123456')
        end

        it 'returns pin number' do
          expect(subject.send_request(:purchase_card, params).response.pin_number).to eq('1234')
        end
      end

      context 'for get_merchants call' do
        it 'returns list of merchants' do
          expect(subject.send_request(:get_merchants, {}).map{ |m| m['Id']}).to eq([22, 12345])
        end
      end

      context 'for invalid params' do
        let(:params) {{ merchant_id: '22', amount: '10.00', username: 'wrong user' }}

        it 'returns unsuccessful result' do
          expect(subject.send_request(:purchase_card, params).successful?).to be_falsey
        end

        it 'returns error' do
          expect(subject.send_request(:purchase_card, params).response).to eq('Unauthorized attempt. Please check credentials and try again.')
        end
      end
    end

    context 'without configs' do
      let(:params) {{ shop_card_id: 3266, to_email: 'rpmilevskiy@gmail.com' }}

      before do
        allow(Rails).to receive(:env).and_return('wrong')
      end

      it 'returns unsuccessful result' do
        expect(subject.send_request(:purchase_card, params).successful?).to be_falsey
      end

      it 'returns error' do
        expect(subject.send_request(:purchase_card, params).response).to eq('Provide configs in buyatabwrapper.yml file')
      end
    end
  end
end
